#include "adc.h"
#include <fsl_interrupt_manager.h>
#include <fsl_clock_manager.h>
#include <string.h>

#ifdef ADC_CALLBACK
	extern void ADC_CALLBACK(void);
#endif

// --------------------------------------------------------------------------
static volatile uint16_t ADC_Value[32];
static volatile uint32_t ADC_done_flag = 0;
static volatile uint32_t ADC_current = 0;
static volatile uint32_t ADC_channels = 0;
static volatile uint32_t ADC_differential = 0;

// --------------------------------------------------------------------------
static void adc_sample_next(uint32_t preincrement)
{
	adc16_chn_config_t cfg;
	cfg.convCompletedIntEnable = true;

	for (ADC_current += preincrement; ADC_current < 32; ADC_current++) {
		if (ADC_channels & (1 << ADC_current))
			break;
	}

	if (ADC_current < 32) {
#if FSL_FEATURE_ADC16_HAS_DIFF_MODE
		cfg.diffConvEnable = !!(ADC_differential & (1 << ADC_current));
#endif
		cfg.chnIdx = (adc16_chn_t)ADC_current;
		ADC16_HAL_ConfigChn(ADC0, 0, &cfg);
		return;
	}

	ADC_current = 0;
	ADC_done_flag = 1;
#if defined(ADC_CALLBACK)
	ADC_CALLBACK();
#endif
}

// --------------------------------------------------------------------------
void ADC0_IRQHandler (void)
{
	if (!ADC16_HAL_GetChnConvCompletedFlag(ADC0, 0))
		return;

	ADC_Value[ADC_current] = ADC16_HAL_GetChnConvValue(ADC0, 0);
	adc_sample_next(1);
}
// --------------------------------------------------------------------------
void init_adc(adc16_clk_divider_t clkDiv, bool lowPower)
{
	adc16_converter_config_t ccfg = (adc16_converter_config_t){
	    .lowPowerEnable = lowPower,
	    .clkDividerMode = clkDiv,
#ifdef ADC_LONG_SAMPLE_TIME
	    .longSampleTimeEnable = true,
		.longSampleCycleMode = ADC_LONG_SAMPLE_TIME,
#else
		.longSampleTimeEnable = false,
		.longSampleCycleMode = kAdc16LongSampleCycleOf24,
#endif
	    .resolution = ADC_RESOLUTION,
	    .clkSrc = kAdc16ClkSrcOfBusClk,
	    .asyncClkEnable = false,
	    .highSpeedEnable = false,
	    .hwTriggerEnable = false,
	    .refVoltSrc = ADC_REFERENCE,
	    .continuousConvEnable = false,
#if FSL_FEATURE_ADC16_HAS_DMA
	    .dmaEnable = false,
#endif
	};

	ADC_done_flag = 0;
	ADC_current = 0;
	ADC_channels = 0;
	ADC_differential = 0;

	CLOCK_SYS_EnableAdcClock(0);
	ADC16_HAL_Init(ADC0);
	ADC16_HAL_ConfigConverter(ADC0, &ccfg);
#if FSL_FEATURE_ADC16_HAS_MUX_SELECT
	ADC16_HAL_SetChnMuxMode(ADC0, kAdc16ChnMuxOfDefault);
#endif
	INT_SYS_EnableIRQ(ADC0_IRQn);
}

// --------------------------------------------------------------------------
void stop_adc(void)
{
	INT_SYS_DisableIRQ(ADC0_IRQn);
	CLOCK_SYS_DisableAdcClock(0);
	ADC_done_flag = 0;
	ADC_current = 0;
	ADC_channels = 0;
	ADC_differential = 0;
}

// --------------------------------------------------------------------------
#if FSL_FEATURE_ADC16_HAS_DIFF_MODE
bool run_adc(uint32_t channels, uint32_t differential)
#else
bool run_adc(uint32_t channels)
#endif
{
	if (ADC_current)
		return false;

	ADC_done_flag = false;
	ADC_channels  = channels;
#if FSL_FEATURE_ADC16_HAS_DIFF_MODE
	ADC_differential = differential;
#endif

	memset((void *)ADC_Value, 0, sizeof(ADC_Value));
	adc_sample_next(0);
	return true;
}

#if FSL_FEATURE_ADC16_HAS_HW_AVERAGE
// --------------------------------------------------------------------------
void set_adc_averaging(bool enable, adc16_hw_average_count_t mode)
{
	adc16_hw_average_config_t cfg = (adc16_hw_average_config_t){
		.hwAverageEnable = enable,
		.hwAverageCountMode = mode,
	};

	ADC16_HAL_ConfigHwAverage(ADC0, &cfg);
}

// --------------------------------------------------------------------------
uint16_t get_adc(uint32_t channel)
{
	return ADC_done_flag ? ADC_Value[channel] : 0;
}

// --------------------------------------------------------------------------
bool get_adc_done(void)
{
	return ADC_done_flag;
}

#endif
