#include "spi.h"
#include <fsl_device_registers.h>
#include <fsl_spi_master_driver.h>
#include <fsl_clock_manager.h>
#include <fsl_interrupt_manager.h>

#if FSL_FEATURE_SPI_16BIT_TRANSFERS
#define SPI_HAL_ReadData SPI_HAL_ReadDataLow
#define SPI_HAL_WriteData SPI_HAL_WriteDataLow
#endif

// --------------------------------------------------------------------------
/*global variable */
volatile SPI_STATE spi;
// --------------------------------------------------------------------------

/********************************************************************
*    Function Name:  SPI_ms_tick                                   *
*    Return Value:   void                                           *
*    Parameters:     void                                           *
*                                                                   *
*    Description:    Decrement counters         	        		*
*    										                        *
********************************************************************/
void SPI_ms_tick(void)
{
	if (spi.tx_tim)
		spi.tx_tim--;
	if (spi.rx_tim)
		spi.rx_tim--;
}
// --------------------------------------------------------------------------

/********************************************************************
*    Function Name:  SPI_Disable                                   *
*    Return Value:   void                                           *
*    Parameters:     void                                           *
*                                                                   *
*    Description:    Disable SPI		         	        		*
*    										                        *
********************************************************************/
void SPI_Disable(void)
{
    SPI_HAL_Init(g_spiBase[SPI_INDEX]);
    INT_SYS_DisableIRQ(g_spiIrqId[SPI_INDEX]);
    CLOCK_SYS_DisableSpiClock(SPI_INDEX);
}
// --------------------------------------------------------------------------

/********************************************************************
*    Function Name:  SPI_Master_Init                   		        *
*    Return Value:   void                                           *
*    Parameters:     void                                           *
*                                                                   *
*    Description:    Initialization SPI mater         	      		*
*    										                        *
********************************************************************/
void SPI_Master_Init(unsigned int freq)
{
	SPI_Type *base = g_spiBase[SPI_INDEX];
	uint32_t sourceClock;
	spi.config = (spi_master_user_config_t){
	  .bitsPerSec = freq,
#if FSL_FEATURE_SPI_16BIT_TRANSFERS
	  .bitCount = kSpi8BitMode,
#endif
#if SPI_COP == 0
	  .polarity = kSpiClockPolarity_ActiveHigh,
#else
	  .polarity = kSpiClockPolarity_ActiveLow,
#endif

#if SPI_COPH == 0
	  .phase = kSpiClockPhase_FirstEdge,
#else
	  .phase = kSpiClockPhase_SecondEdge,
#endif

#if SPI_BIT_ORDER == 0
	  .direction = kSpiMsbFirst,
#else
	  .direction = kSpiLsbFirst,
#endif
	};

    CLOCK_SYS_EnableSpiClock(SPI_INDEX);
    sourceClock = CLOCK_SYS_GetSpiFreq(SPI_INDEX);

    INT_SYS_DisableIRQ(g_spiIrqId[SPI_INDEX]);
    SPI_HAL_Init(base);
    SPI_HAL_SetMasterSlave(base, kSpiMaster);
    SPI_HAL_SetSlaveSelectOutputMode(base, kSpiSlaveSelect_AutomaticOutput);
    SPI_HAL_SetPinMode(base, kSpiPinMode_Normal);

#if FSL_FEATURE_SPI_FIFO_SIZE
    // FIXME: For some reason calling this will fault the MCU
    // SPI_HAL_SetFifoMode(base, false, kSpiTxFifoEmptyClearInt, kSpiRxFifoEmpty);
#endif

    SPI_HAL_Enable(base);

    SPI_HAL_SetBaud(base, spi.config.bitsPerSec, sourceClock);
    SPI_HAL_SetDataFormat(base, spi.config.polarity, spi.config.phase, spi.config.direction);

#if FSL_FEATURE_SPI_16BIT_TRANSFERS
    SPI_HAL_Set8or16BitMode(base, spi.config.bitCount);
#endif
}

// --------------------------------------------------------------------------

/********************************************************************
*    Function Name:  SPI_Write 		                                *
*    Return Value:   void                                           *
*    Parameters:     int			                                *
*                                                                   *
*    Description:    SPI write data		         	        		*
*    										                        *
********************************************************************/
void SPI_Write (unsigned int data)
{
	SPI_Read_Write(data);
}
// --------------------------------------------------------------------------

/********************************************************************
*    Function Name:  SPI_Read 		                                *
*    Return Value:   unsigned int                                   *
*    Parameters:     void			                                *
*                                                                   *
*    Description:    SPI read data		         	        		*
*    										                        *
********************************************************************/
unsigned int SPI_Read(void)
{
	return SPI_Read_Write(0);
}
// --------------------------------------------------------------------------

/********************************************************************
*    Function Name:  SPI_Read_Write 		                                *
*    Return Value:   unsigned int                                   *
*    Parameters:     unsigned int	                                *
*                                                                   *
*    Description:    SPI read write data together  	        		*
*    										                        *
********************************************************************/
inline unsigned int SPI_Read_Write(unsigned int data)
{
	register SPI_Type *base = g_spiBase[SPI_INDEX];
	while (!SPI_HAL_IsTxBuffEmptyPending(base));
	SPI_HAL_WriteData(base, data);
	while (!SPI_HAL_IsReadBuffFullPending(base));
	return SPI_HAL_ReadData(base);
}
// --------------------------------------------------------------------------

/********************************************************************
*    Function Name:  SPI_Write_Fifo		                            *
*    Return Value:   void                                           *
*    Parameters:     int			                                *
*                                                                   *
*    Description:    SPI write data	to mcu fifo without waiting 	*
*    				 for byte reception			   	        		*
*    										                        *
********************************************************************/
void SPI_Write_Fifo (unsigned int data)
{
	register SPI_Type *base = g_spiBase[SPI_INDEX];
	while (!SPI_HAL_IsTxBuffEmptyPending(base));
	SPI_HAL_WriteData(base, data);
}
// --------------------------------------------------------------------------

/********************************************************************
*    Function Name:  SPI_Read_Fifo 		                            *
*    Return Value:   unsigned int                                   *
*    Parameters:     void			                                *
*                                                                   *
*    Description:    SPI read data from the fifo   	        		*
*    										                        *
********************************************************************/
unsigned int SPI_Read_Fifo (void)
{
	register SPI_Type *base = g_spiBase[SPI_INDEX];
	while (!SPI_HAL_IsReadBuffFullPending(base));
	return SPI_HAL_ReadData(base);
}
// --------------------------------------------------------------------------

/********************************************************************
*    Function Name:  SPI_Change_Mode 		                        *
*    Return Value:   void                                   		*
*    Parameters:     SPIMode				                        *
*                                                                   *
*    Description:    Change SPI data transfer format	        	*
*    										                        *
********************************************************************/
void SPI_Change_Mode(unsigned char mode)
{
	switch(mode)
	{
		case SPI_MODE_0:
			spi.config.polarity = kSpiClockPolarity_ActiveHigh;
			spi.config.phase = kSpiClockPhase_FirstEdge;
			break;
		case SPI_MODE_1:
			spi.config.polarity = kSpiClockPolarity_ActiveHigh;
			spi.config.phase = kSpiClockPhase_SecondEdge;
			break;
		case SPI_MODE_2:
			spi.config.polarity = kSpiClockPolarity_ActiveLow;
			spi.config.phase = kSpiClockPhase_FirstEdge;
			break;
		case SPI_MODE_3:
			spi.config.polarity = kSpiClockPolarity_ActiveLow;
			spi.config.phase = kSpiClockPhase_SecondEdge;
			break;
	}
	SPI_HAL_SetDataFormat(g_spiBase[SPI_INDEX], spi.config.polarity, spi.config.phase, spi.config.direction);
}

#if FSL_FEATURE_SPI_16BIT_TRANSFERS
#undef SPI_HAL_ReadData
#undef SPI_HAL_WriteData
#endif
